use async_channel::{Receiver, Sender};
use tokio::runtime::Runtime;
use xmpp::{BareJid, ClientBuilder, ClientFeature, Event};

use std::sync::OnceLock;

#[derive(Clone, Debug)]
pub enum XMPPCommand {
    SendPM(BareJid, String),
}

fn tokio_runtime() -> &'static Runtime {
    static RUNTIME: OnceLock<Runtime> = OnceLock::new();
    RUNTIME.get_or_init(|| Runtime::new().expect("Setting up tokio runtime needs to succeed."))
}

pub(crate) fn client(jid: &str, password: &str) -> (Receiver<xmpp::Event>, Sender<XMPPCommand>) {
    let (event_sender, event_receiver) = async_channel::bounded::<xmpp::Event>(1);
    let (cmd_sender, cmd_receiver) = async_channel::bounded::<XMPPCommand>(1);

    let jid = jid.to_string();
    let password = password.to_string();

    tokio_runtime().spawn(async move {
        let mut client = ClientBuilder::new(BareJid::new(&jid).unwrap(), &password)
            .set_default_nick("xmpp-client")
            .enable_feature(ClientFeature::ContactList)
            .build();

        loop {
            tokio::select! {
                Some(events) = client.wait_for_events() => {
                    for event in events {
                        match event {
                            Event::ContactAdded(_) => {
                                event_sender.send(event).await.expect("BOOOOOOHOOOO");
                            }
                            Event::ChatMessage(_, _, _, _) => {
                                event_sender.send(event).await.expect("BOOOHOOOO");
                            }
                            _ => {
                                continue;
                            }
                        }
                    }
                }

                command = cmd_receiver.recv() => {
                    match command.unwrap() {
                        XMPPCommand::SendPM(jid, content) => {
                            client.send_message(jid.into(), xmpp::parsers::message::MessageType::Chat, "en", &content).await;
                        },
                    }
                }
            }
        }
    });

    (event_receiver, cmd_sender)
}
